import sys
import KNode
from delocate import *
from connect import push,pull,remove
from datetime import datetime

class shell:

    VALID_CMD = [ \
        'pwd', \
        'ls', \
        'cd', \
        'mkdir', \
        'cp', \
        'mv', \
        'rm', \
        'cat', \
	'down',\
	'mkfile',\
        'usage',\
        'cmds',\
        'quit',\
	"changeperms"]

    USAGE = {
        'usage': 'To get help with a command, type "usage command".'+\
        ' To see a list of commands, type "cmds"',\
        'quit' : 'quit\nExit the kbox shell.\nType "quit".',\
        'cmds': 'cmds\nShows a list of valid commands.\nType "cmds"',\
        'pwd' : 'pwd\nPrint working directory. Takes no arguments.'+\
        '\nType "pwd"',\
        'ls' : 'ls\nList the files and/or directories in the current'+\
        ' directory. Takes an optional argument of the path to the '+\
        'directory in which you wish to list the contents.\nType "ls"'+\
        ' or "ls <TARGET_DIRECTORY>"',\
        'cd' : 'cd\nChange directory. ".." represents the parent '+\
        'directory and "." represents the current directory.\nType'+\
        ' "cd <TARGET_DIRECTORY>" where <TARGET_DIRECTORY> is the '+\
        'directory to which you wish to move.',\
        'mkdir' : 'mkdir\nMake a new directory.\nType "mkdir '+\
        '<new_directory_name>"',\
        'cp' : 'cp\nCopy a file or directory.\nType "cp <file_to_copy>'+\
        ' <location_to_copy_to>"', \
        'mv' : 'mv\nMove a file or directory.\nType "mv <file_to_move> '+\
        '<location_to_move_to>"', \
        'rm' : 'rm\nRemove a file or directory.\nType "rm <filename>" '+\
        'with an optional "-R" flag to indicate to delete recursively.', \
        'cat' : 'cat\nPrint the contents of a file to the shell.\nType'+\
        ' "cat <filename>"', \
	'down' : 'down\nDownload a file to your local file system.\nType'+\
        ' "down <kbox_path> <local_path>"',\
	'mkfile' : 'mkfile\nMake a file from an existing file in your '+\
        'local file system and push it to the KBox server.\nType '+\
        '"mkfile <filename>"',\
	"changeperms" : 'changeperms\nChange the permissions on a file'+\
        '\nType "changeperms <kbox_path> <read_or_write> <local_path_to_perm>"'
        }

    ERR = {
        'PATH_NOT_FOUND': 'The system could not find the specified path',
        'CMD_NOT_FOUND': 'Command not recognized',
        'SYNTAX_ERROR': 'Syntax error',
        'UNIMPLEMENTED': 'This has not been implemented'
        }

    def __init__(self, root_node, priv_key):

        self.PRIV_KEY = priv_key

        self.ROOT = root_node
        self.CUR_NODE = root_node
        self.PROMPT = root_node.name
        self.ROOT.populate(self.PRIV_KEY)

    def try_move(self, path, dir_only = False):
        path_list = path.split('/')
        loc = None
        new_path = None

        if path_list[0] == '':
            loc = self.ROOT
            new_path = self.ROOT.name
            path = path[1:]
        else:
            loc = self.CUR_NODE
            new_path = self.PROMPT
                
        for comp in path_list:
            if comp == '.':
                pass
            elif comp == '..':
                if loc.parent:
                    loc = loc.parent
                    new_path = "/".join(new_path.split("/")[:-1])
                else:
                    print >> sys.stderr, self.ERR['PATH_NOT_FOUND'] + ': ' + path

            else:
                children = [child.name for child in loc.contents]
                if comp in children:
                    idx = children.index(comp)
                    if dir_only and (not loc.contents[idx].q):
                         print >> sys.stderr, path + ' is a file, not a directory'
                    else: 
                        loc = loc.contents[idx]
                        new_path += '/' + comp
                else:
                    print >> sys.stderr, self.ERR['PATH_NOT_FOUND'] + ': ' + path

        return loc, new_path

    ## cmds
    def handle_cmds(self, *args):
        return '\n'.join(cmd for cmd in self.VALID_CMD)

    ## usage (<cmd>)
    def handle_usage(self, *args):
        cmd = args[0] if len(args) > 0 else 'usage'
        if cmd in self.USAGE:
            return self.USAGE[cmd]
        else:
            print >> sys.stderr, self.ERR['CMD_NOT_FOUND'] + ": " + cmd

    ## pwd
    def handle_pwd(self, *args):
        return self.CUR_NODE.name

    ## ls or ls <some_directory>
    def handle_ls(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path)
        else:
            loc = self.CUR_NODE
            
        if loc:
            if loc.q:
                return '\n'.join(child.name for child in loc.contents)
            else:
                return loc.name
    ## cd <some_directory>
    def handle_cd(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path, dir_only=True)
            if loc:
                self.CUR_NODE = loc
                self.PROMPT = path
    ## mkdir <new_directory>
    def handle_mkdir(self, *args):
        name = args[0]
        delocate_dir(name, self.CUR_NODE, self.PRIV_KEY)

    ## cp <file_to_copy> <location_to_copy_to>
    ###copy does not break links, it symbolically copies                                       
    def handle_cp(self, *args):
        to_copy=args[0] if len(args)==2 else None
        to_location=args[1] if len(args)==2 else None
        if to_copy==None or to_location==None:
            print "usage: cp [file to copy] [location to copy to]"
            return

        startNode=self.CUR_NODE
        to_copy_node, path=self.try_move(to_copy,dir_only=True)

        if to_copy_node!=startNode:
            to_copy_text=to_copy_node.this_to_text()
            f=open("/tmp/cpbuffer","w")
            f.write(to_copy_text)
            f.close()
        else:
            print "file you are trying to copy does not exist"
            return

        to_location_node, path=self.try_move(to_location)
        if to_location_node!=startNode:
            self.CUR_NODE=to_location_node
            self.handle_mkfile(*["/tmp/cpbuffer","-R"])
        else:
            print "copy to location does not exist"
            return
        try:
            os.remove("/tmp/cpbuffer")
        except:
            pass
        self.CUR_NODE=startNode

    def handle_mv(self, *args):
        pass

    ## rm <filename> (-r)
    def handle_rm(self, *args):
        if len(args) == 0:
            print >> sys.stderr, "Need to specify a filename to remove"
            return
        elif len(args) == 1:
            path = args[0]
            file_to_remove, path = self.try_move(path)
            parent_node = file_to_remove.parent

            parent_node.remove_child(file_to_remove, self.PRIV_KEY)

            remove(file_to_remove.to_cipher(), self.PRIV_KEY)
        elif len(args) == 2:
            if args[0] == "-R" or args[0] == "-r":
                path = args[1]
            elif args[1] == "-R" or args[1] == "-r":
                path = args[0]
            else:
                print >> sys.stderr, self.ERR['SYNTAX_ERROR']
                return
            
            file_to_remove, temp_path = self.try_move(path)

            if file_to_remove.q == 1:
                for knode in file_to_remove.contents:
                    self.handle_rm("-r", path + "/" + knode.name)
            
            parent_node = file_to_remove.parent

            if parent_node:
                parent_node.remove_child(file_to_remove, self.PRIV_KEY)
            
            remove(file_to_remove.to_cipher(), self.PRIV_KEY)
            
        
            
    ## cat <filename>
    def handle_cat(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path)
        else:
            loc = CUR_NODE

        if loc.contents == None:
        	loc.fillup()
        return loc.contents

    def get_perm_list(self, os_path):
        os_path = os.path.realpath(os_path)
        if os.path.isfile(os_path):
            perm_file = open(os_path, 'r')
            perm_list = [line.rstrip().decode("hex") for line in perm_file]
            perm_file.close()
        return perm_list

    ## changeperms <kbox_path> <read_or_write> <local_path_to_perm>
    def handle_changeperms(self, *args):
        path = args[0]
        rw = args[1]
        perm_path = args[2]

        perm_list = self.get_perm_list(perm_path)

        if (rw == '-R') or (rw == '--read'):
            loc, path = self.try_move(path)

            new_key =  hashlib.sha256(os.urandom(64)).hexdigest()
            loc.set_perm(perm_list, new_key, self.PRIV_KEY)
        elif (rw == '-W') or (rw == '--write'):
            print >> sys.stderr, self.ERR['UNIMPLEMENTED']
        else:
            print >> sys.stderr, self.ERR['SYNTAX_ERROR']

    ## mkfile <filename>
    def handle_mkfile(self, *args):
        os_path = args[0]
        remote = args[1] if len(args) > 1 else None

        if remote is None:
            delocate_file(os_path, self.CUR_NODE, self.PRIV_KEY)
        elif (remote == '-R') or (remote == '--remote'):
            os_path = os.path.realpath(os_path)
            if os.path.isfile(os_path):
                node_file = open(os_path, 'r')
                strings = [line for line in node_file]
                node_file.close()

                new_node = KNode(strings)

		new_node.timestamp = datetime.now()	
		
                new_node.parent=self.CUR_NODE

		new_node.populate(self.PRIV_KEY)
		
                self.CUR_NODE.contents.append(new_node)

                push(self.CUR_NODE.to_cipher(), \
                     self.CUR_NODE.to_text(), \
                     self.CUR_NODE.getKey(), \
                     "", \
                     self.PRIV_KEY)            
        else:
            print >> sys.stderr, self.ERR['SYNTAX_ERROR']

    ## down <kbox_path> <local_path>
    def handle_down(self, *args):
        path = args[0]
        os_path = args[1]
	
	loc, path = self.try_move(path)
	os_path = os.path.realpath(os_path)
        if not os.path.isfile(os_path):
            wfile = open(os_path, 'w')
	    wfile.write(loc.this_to_text())
	    wfile.close()
		 
    
    def parse_cmd(self, cmd):
        cmd_args = cmd.split()

        cmd = cmd_args[0]
        args = ','.join('\'' + arg + '\'' \
                        for arg in cmd_args[1:])
        return cmd, args

    def kbox_handle(self, cmd):
        cmd, args = self.parse_cmd(cmd)

        if cmd in self.VALID_CMD:
            try:
                cmd_str = 'self.handle_' + cmd + '(' + args + ')'
                out = eval(cmd_str)
                if out:
                    print out
            except Exception as e:
                print >> sys.stderr, self.ERR['SYNTAX_ERROR']
                print >> sys.stderr, e
                print "Try 'usage " + cmd + "'"
        else:
            print >> sys.stderr, self.ERR['CMD_NOT_FOUND'] + ": " + cmd

    def run(self):
        
        cmd = raw_input(self.PROMPT + '$ ')
        while cmd != 'quit' and cmd != 'q':
            self.kbox_handle(cmd)
            cmd = raw_input(self.PROMPT + '$ ')




