import sys
import KNode
from delocate import *
from connect import push,pull

class shell:

    VALID_CMD = [ \
        'pwd', \
        'ls', \
        'cd', \
        'mkdir', \
        'rmdir', \
        'cp', \
        'mv', \
        'rm', \
        'cat', \
        'usage']

    USAGE = {
        'usage': 'To get help with a command, type \'usage command\''
        }

    ERR = {
        'PATH_NOT_FOUND': 'The system could not find the specified path',
        'CMD_NOT_FOUND': 'Command not recognized',
        'SYNTAX_ERROR': 'Syntax error'
        }

    def __init__(self,root):
        self.ROOT = root
        self.CUR_NODE = root
        self.PROMPT = root.name

    def try_move(self, path, dir_only = False):
        path_list = path.split('/')
        loc = None
        new_path = None

        if path_list[0] == '':
            loc = self.ROOT
            new_path = self.ROOT.name
            path = path[1:]
        else:
            loc = self.CUR_NODE
            new_path = self.PROMPT
                
        for comp in path_list:
            if comp == '.':
                pass
            elif comp == '..':
                if loc.parent:
                    loc = loc.parent
                    new_path = new_path[:-1]
                else:
                    print >> sys.stderr, err['PATH_NOT_FOUND'] + ': ' + path

            else:
                children = [child.name for child in loc.contents]
                if comp in children:
                    idx = children.index(comp)
                    if dir_only and (not loc.contents[idx].q):
                         print >> sys.stderr, path + ' is a file, not a directory'
                    else: 
                        loc = loc.contents[idx]
                        new_path += '/' + comp
                else:
                    print >> sys.stderr, ERR['PATH_NOT_FOUND'] + ': ' + path

        return loc, new_path

    def handle_usage(self, *args):
        cmd = args[0] if len(args) > 0 else 'usage'
        if cmd in self.USAGE:
            return self.USAGE[cmd]
        else:
            print >> sys.stderr, self.ERR['CMD_NOT_FOUND'] + ": " + cmd

    def handle_pwd(self, *args):
        return self.CUR_NODE.name

    def handle_ls(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path)
        else:
            loc = self.CUR_NODE
            
        if loc:
            if loc.q:
                return '\n'.join(child.name for child in loc.contents)
            else:
                return loc.name

    def handle_cd(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path, dir_only=True)
            if loc:
                self.CUR_NODE = loc
                self.PROMPT = path

    def handle_mkdir(self, *args):
        

    def handle_rmdir(self, *args):
        pass

    def handle_cp(self, *args):
        pass

    def handle_mv(self, *args):
        pass

    def handle_rm(self, *args):
        pass

    def handle_cat(self, *args):
        if len(args) > 0:
            path = args[0]
            loc, path = self.try_move(path)
        else:
            loc = CUR_NODE

        loc.fill_up()
        return loc.contents

    def parse_cmd(self, cmd):
        cmd_args = cmd.split()

        cmd = cmd_args[0]
        args = ','.join('\'' + arg.strip('-').strip('--') + '\'' \
                        for arg in cmd_args[1:])
        return cmd, args

    def kbox_handle(self, cmd):
        cmd, args = self.parse_cmd(cmd)

        if cmd in self.VALID_CMD:
            try:
                cmd_str = 'self.handle_' + cmd + '(' + args + ')'
                print eval(cmd_str)
            except Exception as e:
                print >> sys.stderr, e.message
                print "Try 'usage " + cmd + "'"
        else:
            print >> sys.stderr, self.ERR['CMD_NOT_FOUND'] + ": " + cmd

    def run(self):
        
        cmd = raw_input(self.PROMPT + '$ ')
        while cmd != ('quit' or 'q'):
            self.kbox_handle(cmd)
            cmd = raw_input(self.PROMPT + '$ ')

    
import random

class basic_node:
    def __init__(self, name, q):
        self.name = name
        self.q = q
        self.parent = None
        self.contents = []

    def add_parent(self, node):
        self.parent = node
        if self not in node.contents:
            node.contents.append(self)

    def add_child(self, node):
        self.contents.append(node)
        if node.parent != self:
            node.add_parent(self)

def build_tree():
    root = basic_node('root', 1)

    for i in xrange(5):
        ni = basic_node('foo' + str(i), 1)
        root.add_child(ni)
        for j in xrange(6):
            r = random.random()
            if r > 0.5:
                nij = basic_node('foo' + str(i) + str(j), 0)
                ni.add_child(nij)

    return root
